/*
 * Copyright  2008 Nokia Corporation.
 */

#ifndef __MOBILEPAINTAPPUI_h__
#define __MOBILEPAINTAPPUI_h__

// INCLUDES
#include <aknappui.h>
#include <aknutils.h>
#include <ImageConversion.h>

// FORWARD DECLARATIONS
class CMobilePaintAppView;
class CSvgHandler;
class CSplashScreenDialog;

const TUid KUidHelpFile = {0x2000E195};  // From help


class MSaveImageObserver
    {
    public:
        virtual void ImageSaved(TInt aError)=0;
    };

class CSaveImageAO : public CActive
    {
    public:
        static CSaveImageAO* NewL(MSaveImageObserver* aObserver);
        static CSaveImageAO* NewLC(MSaveImageObserver* aObserver);
        virtual ~CSaveImageAO();

    public:
        void SaveAsL(CFbsBitmap* aBitmap, TFileName& aFileName, TUid aImageType, TUid aImageSubType);
        void DoCancel();
        void RunL();
        TInt RunError(TInt aError);
        void GetEncoderImageTypesL(RImageTypeDescriptionArray& aImageTypeArray);
        
    private:
        void ConstructL();
        CSaveImageAO(MSaveImageObserver* aObserver);
        
    private:
        MSaveImageObserver*     iObserver;
        CImageEncoder*          iImageEncoder;
        RFs                     iFs;
    };


// CLASS DECLARATION
class CMobilePaintAppUi : public CAknAppUi, public MSaveImageObserver
    {
    public:
        void ConstructL ( );
        CMobilePaintAppUi ( );
        virtual ~CMobilePaintAppUi ( );
        
        TBool IsOptionsButtonOnTop();
        void SearchOptionsButtonPosition();
        void SearchCbaPosition();
        void HandleCommandL (TInt aCommand );
        AknLayoutUtils::TAknCbaLocation CbaPosition();
        inline CSvgHandler* SvgHandler(){return iSvgHandler;};

    private:
        // Functions from base classes
        void HandleResourceChangeL( TInt aType );
        CArrayFix<TCoeHelpContext>* HelpContextL() const;

    private: // From MSaveImageObserver
        void ImageSaved(TInt aError);
        
    private:
        void SaveAsL();
        
    private:
        CMobilePaintAppView*                    iAppView;
        TBool                                   iOptionButtonOnTop;
        AknLayoutUtils::TAknCbaLocation         iCbaPosition;
        CSvgHandler*                            iSvgHandler;
        CSplashScreenDialog*                    iSplashScreenDialog;
        CSaveImageAO*                           iSaveAsAO;
    };

#endif // __MOBILEPAINTAPPUI_h__

