/*
 * Copyright  2008 Nokia Corporation.
 */

#ifndef TOOLBARCONTROL_H
#define TOOLBARCONTROL_H

// INCLUDES
#include <e32std.h>
#include <e32base.h>
#include <coecntrl.h>
#include "mobilepaintappview.h"
#include <touchfeedback.h>
#include <touchlogicalfeedback.h>

// How far stylus can go from toolbar while dragging that it is accepted 
const TInt KDraggingAcceptGrow = 40;

// Toolbar button width
const TInt KToolbarItemWidth = 45;

// Toolbar button height
const TInt KToolbarItemHeight = 45;

// Amount of buttons in the toolbar
const TInt KToolbarItemCount = 8;

// Gap between buttons in the toolbar
const TInt KToolbarItemGap = 2;

// How close from screen borders toolbar can be dragged after it locks into screen borders
const TInt KToolbarGap = 5; 

// How much toolbar must move after it is regonized as dragging. This value
// helps selecting to button from toolbar because toolbar can move a little while selecting/tapping.
const TInt KDraggingAccepted = 8;

// Toolbar width in when toolbar is in portrait mode
const TInt KPortraitToolbarWidth = 60;

// Toolbar height in when toolbar is in landscape mode (1 row of buttons)
const TInt KLandscapeToolbarHeight = 60;

// Toolbar height in when toolbar is in landscape mode (2 rows of buttons)
const TInt KLandscapeToolbarHeight2Rows = KToolbarItemHeight*2 + KToolbarItemGap + (KPortraitToolbarWidth - KToolbarItemWidth);

// Toolbar height in when toolbar is in portrait mode (2 rows of buttons)
const TInt KPortraitToolbarWidth2Rows = KToolbarItemWidth*2 + KToolbarItemGap + (KPortraitToolbarWidth - KToolbarItemWidth);

const TInt KCBAPortraitWidth = 360;
const TInt KCBAPortraitHeight = 60;
const TInt KCBALandscapeWidth = 60;
const TInt KCBALandscapeHeight = 360;

const TInt KMAXPenCount = 3;
const TInt KMAXPenSize = 6;
const TInt KMINPenSize = 2;
const TInt KSizebuttonDownTime(500000);

enum TTool
    {
    ENoTool = 0,
    ENew,
    EPencil,
    ELine,
    ECircle,
    ERect,
    EColor,
    EUndo,
    EOptions,
    EExit
    };

class CMobilePaintAppUi;
// CLASS DECLARATION
class CToolbarControl : public CCoeControl
    {
    public:
        ~CToolbarControl ( );
        static CToolbarControl* NewL (CMobilePaintAppView* aParent, TRect aRect);
        static CToolbarControl* NewLC (CMobilePaintAppView* aParent, TRect aRect);

    private:
        void Draw(const TRect& aRect) const;
        void SizeChanged();
        void HandlePointerEventL(const TPointerEvent& aPointerEvent);
        void Move(const TPoint& aPoint);
        TBool IsDragDone(const TPointerEvent& aPointerEvent);
        void ShowUnpressedSizeButtonL();
        static TInt SizeButtonUp(TAny* aAny);
        void DrawSizeButton() const;

        
    public:
        void SelectSizeL();
        void SelectColourL();
        inline TTool ActiveTool(){return iTool;};
        inline void SetActiveTool(TTool aTool){iTool = aTool;};
        inline RDrawableWindow* ParentDrawableWindow(){return iParent->DrawableWindow();};
        
        void SetTop();
        void SetBottom();
        void SetLeft();
        void SetRight();
        
        
    private:
        CToolbarControl (CMobilePaintAppView* aParent);
        void ConstructL (CMobilePaintAppView* aParent, TRect aRect);
        
    private:
        TTool                   iTool;
        CMobilePaintAppView*    iParent;
        TPoint                  iMovePoint;
        TPoint                  iDragStartPoint;
        
        TRect                   iNewPictureItem;
        TRect                   iPencilItem;
        TRect                   iLineItem;
        TRect                   iCircleItem;
        TRect                   iRectangleItem;
        TRect                   iColorItem;
        TRect                   iSizeItem;
        TRect                   iUndoItem;
        
        TBool                   iDrag;
        
        MTouchFeedback*         iTouchFeedBack;
        CMobilePaintAppUi*      iAppUi;
        
        TInt                    iPenSize;
        TInt                    iPenButtonIndex;
        TBool                   iPenButtonIsUp;
        CPeriodic*              iPeriodic;
        
    };

#endif // TOOLBARCONTROL_H
