/*
 * Copyright  2008 Nokia Corporation.
 */

// INCLUDE FILES
#include <avkon.hrh>
#include <aknmessagequerydialog.h>
#include <MobilePaint_0xA000A332.rsg>
#include <aknquerydialog.h>
#include <pathinfo.h>
#include <hlplch.h>

#include "MobilePaint.hrh"
#include "MobilePaint.pan"
#include "MobilePaintApplication.h"
#include "MobilePaintAppUi.h"
#include "MobilePaintAppView.h"
#include "csvghandler.h"
#include "csplashscreendialog.h"
#include "MobilePaint.hlp.hrh"

// We specify a custom cleanup method for the RImageTypeDescriptionArray.
// This way we can, in that method, 
// simply call CleanupStack::PopAndDestroy() on the array and do whatever 
// additional cleanup we need here. In this case ResetAndDestroy() is called 
// on the array to release resources consumed by it.
// In general, creating a custom cleanup method allows cleanup to be more 
// sophisticated than simply deleting objects, for example, releasing access to 
// some shared resource.
void CleanupRArray( TAny* object )
    {
    ((RImageTypeDescriptionArray*)object)->ResetAndDestroy();
    }



CSaveImageAO* CSaveImageAO::NewL(MSaveImageObserver* aObserver)
    {
    CSaveImageAO* self = CSaveImageAO::NewLC(aObserver );
    CleanupStack::Pop (self );
    return self;
    }

CSaveImageAO* CSaveImageAO::NewLC(MSaveImageObserver* aObserver)
    {
    CSaveImageAO* self = new ( ELeave ) CSaveImageAO(aObserver);
    CleanupStack::PushL (self );
    self->ConstructL();
    return self;
    }

CSaveImageAO::~CSaveImageAO()
    {
    Cancel();
    delete iImageEncoder;
    iFs.Close();
    }

void CSaveImageAO::SaveAsL(CFbsBitmap* aBitmap, TFileName& aFileName, TUid aImageType, TUid aImageSubType)
    {
    if (!IsActive())
        {
        if (iImageEncoder)
            {
            delete iImageEncoder; 
            iImageEncoder = NULL;
            }

        iImageEncoder = CImageEncoder::FileNewL( iFs, aFileName, 
            CImageEncoder::EOptionNone, aImageType, aImageSubType );
        
        iImageEncoder->Convert( &iStatus, *aBitmap );
        SetActive();
        }
    }

void CSaveImageAO::DoCancel()
    {
    iImageEncoder->Cancel();
    }

void CSaveImageAO::RunL()
    {
    iObserver->ImageSaved(iStatus.Int());
    }

TInt CSaveImageAO::RunError(TInt aError)
    {
    iObserver->ImageSaved(aError);
    return KErrNone;
    }
    
void CSaveImageAO::ConstructL()
    {
    CActiveScheduler::Add(this);
    User::LeaveIfError( iFs.Connect() );
    }

CSaveImageAO::CSaveImageAO(MSaveImageObserver* aObserver)
: CActive(EPriorityStandard), iObserver(aObserver)
    {
    }

void CSaveImageAO::GetEncoderImageTypesL( 
    RImageTypeDescriptionArray& aImageTypeArray )
    {
    CImageEncoder::GetImageTypesL( aImageTypeArray );
    }






// -----------------------------------------------------------------------------
// CMobilePaintAppUi::ConstructL()
// Symbian 2nd phase constructor can leave.
// -----------------------------------------------------------------------------
//
void CMobilePaintAppUi::ConstructL ( )
    {
    // Initialise app UI with standard value.
    BaseConstructL (EAknEnableSkin | EAknTouchCompatible);

    SearchCbaPosition();
    SearchOptionsButtonPosition();
    
    iSvgHandler = CSvgHandler::NewL();
    
    // Create view object
    iAppView = CMobilePaintAppView::NewL (ClientRect ( ) );
    AddToStackL( iAppView );
    
    // Show splash screen
    iSplashScreenDialog = new (ELeave) CSplashScreenDialog();
    iSplashScreenDialog->ExecuteLD(R_SPLASH_SCREEN_DIALOG);
    iSplashScreenDialog = NULL;
    }

// -----------------------------------------------------------------------------
// CMobilePaintAppUi::CMobilePaintAppUi()
// C++ default constructor can NOT contain any code, that might leave.
// -----------------------------------------------------------------------------
//
CMobilePaintAppUi::CMobilePaintAppUi ( )
    {
    // No implementation required
    }

// -----------------------------------------------------------------------------
// CMobilePaintAppUi::~CMobilePaintAppUi()
// Destructor.
// -----------------------------------------------------------------------------
//
CMobilePaintAppUi::~CMobilePaintAppUi ( )
    {
    if (iAppView )
        {
        RemoveFromStack( iAppView );
        delete iAppView;
        delete iSvgHandler;
        delete iSplashScreenDialog;
        delete iSaveAsAO;
        }
    }

// -----------------------------------------------------------------------------
// CMobilePaintAppUi::HandleCommandL()
// Takes care of command handling.
// -----------------------------------------------------------------------------
//
void CMobilePaintAppUi::HandleCommandL (TInt aCommand )
    {
    switch (aCommand )
        {
        case EEikCmdExit:
        case EAknSoftkeyExit:
            {
            Exit ( );
            break;
            }
        case ESaveAs:
            {
            SaveAsL();
            break;
            }
        case EAbout:
            {
            CAknMessageQueryDialog* dlg = new (ELeave)CAknMessageQueryDialog();
            dlg->PrepareLC (R_ABOUT_QUERY_DIALOG );
            HBufC* title = iEikonEnv->AllocReadResourceLC (R_ABOUT_DIALOG_TITLE );
            dlg->QueryHeading()->SetTextL (*title );
            CleanupStack::PopAndDestroy ( ); //title
            HBufC* msg = iEikonEnv->AllocReadResourceLC (R_ABOUT_DIALOG_TEXT );
            dlg->SetMessageTextL (*msg );
            CleanupStack::PopAndDestroy ( ); //msg
            dlg->RunLD ( );
            break;
            }
        case EHelp:
            {
            CArrayFix<TCoeHelpContext>* buf = CCoeAppUi::AppHelpContextL ( );
            HlpLauncher::LaunchHelpApplicationL (iEikonEnv->WsSession ( ), buf );
            break;
            }
        default:
            {
            Panic (EMobilePaintUi );
            break;
            }
        };
    }

void CMobilePaintAppUi::SaveAsL()
    {
    // Create path for filename
    TFileName path = PathInfo::PhoneMemoryRootPath(); 
    path.Append(PathInfo::ImagesPath());
    
    // Ask filename
    CAknTextQueryDialog* dlg = CAknTextQueryDialog::NewL(path,CAknQueryDialog::ENoTone);
    dlg->PrepareLC(R_SAVE_DIALOG);
    dlg->SetPromptL(_L("Save As"));
    if(dlg->RunLD())
        {
        // Cut image type e.g. ".gif" if user gave it
        if (path.Find(_L(".")) > -1)
            {
            path.SetLength(path.Find(_L(".")));
            }
        // Use .jpeg image type
        path.Append(_L(".jpeg"));
        
        // Start to encode image to file
        if (!iSaveAsAO)
            {
            iSaveAsAO = CSaveImageAO::NewL(this);
            }
        
        RImageTypeDescriptionArray imageTypes; 
        iSaveAsAO->GetEncoderImageTypesL( imageTypes);
        // Create a custom cleanup item and specify CleanupRArray to be the
        // method that is called once we call CleanupStack::PopAndDestroy() at the
        // end of this method.
        CleanupStack::PushL( TCleanupItem(CleanupRArray, &imageTypes) );
        
        CImageTypeDescription* type = NULL;
        
        for (TInt i=0;i<imageTypes.Count();i++)
            {
            type = imageTypes[i];
            if (type->Description().Compare(_L("jpeg")))
                {
                break;
                }
            type = NULL;
            }

        if (type)
            {
            iSaveAsAO->SaveAsL(iAppView->BackBuffer(),path,type->ImageType(),type->SubType());
            }
        else
            {
            iEikonEnv->AlertWin(_L("MobilePaint"),_L("No .jpeg image type support found from terminal, saving failed!"));
            }

        CleanupStack::PopAndDestroy(); // array
        }
    }

void CMobilePaintAppUi::ImageSaved(TInt aError)
    {
    if (aError)
        {
        iEikonEnv->AlertWin(_L("MobilePaint"),_L("Saving image failed!"));
        }
    else
        {
        iEikonEnv->InfoMsg(_L("Image saved"));
        }
    }


void CMobilePaintAppUi::HandleResourceChangeL( TInt aType )
    {
    CAknAppUi::HandleResourceChangeL( aType );
       
    if ( aType==KEikDynamicLayoutVariantSwitch )
        {
        SearchCbaPosition();
        SearchOptionsButtonPosition();
        
        if (iSplashScreenDialog)
            {
            TRect rect;
            AknLayoutUtils::LayoutMetricsRect(AknLayoutUtils::EScreen,rect);
            iSplashScreenDialog->SetRect(rect);
            }
        }   
    }


TBool CMobilePaintAppUi::IsOptionsButtonOnTop()
    {
    return iOptionButtonOnTop;
    }

void CMobilePaintAppUi::SearchOptionsButtonPosition()
    {
    iOptionButtonOnTop = EFalse;
    CEikButtonGroupContainer* cba = Cba();
    if(!cba)
        {
        return;
        }
    CCoeControl* options = cba->ControlOrNull(EAknSoftkeyOptions);
    CCoeControl* exit = cba->ControlOrNull(EAknSoftkeyExit);
    if( options && exit )
        {
        if( options->Position().iY < exit->Position().iY )
            {
            iOptionButtonOnTop = ETrue;
            }
        }
    }

AknLayoutUtils::TAknCbaLocation CMobilePaintAppUi::CbaPosition()
    {
    return iCbaPosition;
    }

void CMobilePaintAppUi::SearchCbaPosition()
    {
    // This does not work in S60 5th v0.9 SDK
    //iCbaPosition = AknLayoutUtils::CbaLocation();
    
    // Searching CBA locations
    CEikButtonGroupContainer* cba = Cba();
    if(cba)
        {
        CCoeControl* options = cba->ControlOrNull(EAknSoftkeyOptions);
        if (options)
            {
            if (options->Rect().iTl.iX > ApplicationRect().Width()/2)
                {
                iCbaPosition = AknLayoutUtils::EAknCbaLocationRight;
                }
            else
                {
                iCbaPosition = AknLayoutUtils::EAknCbaLocationBottom;
                }
            }
        }
    }

CArrayFix<TCoeHelpContext>* CMobilePaintAppUi::HelpContextL() const
    {
    CArrayFixFlat<TCoeHelpContext>* array = new(ELeave)CArrayFixFlat<TCoeHelpContext>(1);
    CleanupStack::PushL (array );
    array->AppendL (TCoeHelpContext (KUidHelpFile, KGeneral_Information ) );
    CleanupStack::Pop (array );
    return array;
    }


// End of File

