/*
 * Copyright  2008 Nokia Corporation.
 */

#include "ctoolbarcontrol.h"
#include "MobilePaintAppView.h"
#include "MobilePaintAppUi.h"
#include "cdrawingengine.h"
#include "csvghandler.h"
#include <eikenv.h>
#include <akncolourselectiongrid.h>
#include <MobilePaint_0xA000A332.rsg>


CToolbarControl::CToolbarControl (CMobilePaintAppView* aParent) : iParent(aParent)
    {
    // No implementation required
    }

CToolbarControl::~CToolbarControl ( )
    {
    if (iPeriodic)
        {
        iPeriodic->Cancel();
        delete iPeriodic;
        }
    }

CToolbarControl* CToolbarControl::NewLC (CMobilePaintAppView* aParent, TRect aRect)
    {
    CToolbarControl* self = new (ELeave)CToolbarControl(aParent);
    CleanupStack::PushL (self );
    self->ConstructL (aParent,aRect);
    return self;
    }

CToolbarControl* CToolbarControl::NewL (CMobilePaintAppView* aParent, TRect aRect)
    {
    CToolbarControl* self=CToolbarControl::NewLC (aParent,aRect);
    CleanupStack::Pop ( ); // self;
    return self;
    }

void CToolbarControl::ConstructL (CMobilePaintAppView* aParent, TRect /*aRect*/)
    {
    SetContainerWindowL(static_cast<CCoeControl&>(*aParent));
    
    iAppUi = (static_cast<CMobilePaintAppUi*>(iEikonEnv->AppUi()));

    // Touch feedback
    iTouchFeedBack = MTouchFeedback::Instance();
    iTouchFeedBack->SetFeedbackEnabledForThisApp(ETrue);

    // Default toolbar values
    iPenSize = KMINPenSize;
    iPenButtonIndex = 1;
    iPenButtonIsUp = ETrue;
    SetActiveTool(EPencil);
    }

void CToolbarControl::Draw( const TRect& /*aRect*/ ) const
    {
    // Get the standard graphics context
    CWindowGc& gc = SystemGc();

    TBool invertMask = EFalse;

    
    // Background of the toolbar -----------------------------------
    TRect toolbarRect = TRect(TPoint(0,0),Size());

    if (iAppUi->CbaPosition() == AknLayoutUtils::EAknCbaLocationBottom)
        {
        // When terminal is in portrait mode
        if (Size().iHeight > Size().iWidth)
            {
            // Toolbar is in portrait mode
            gc.BitBltMasked(Rect().iTl, iAppUi->SvgHandler()->iBarPortraitBitmap, 
                    toolbarRect, iAppUi->SvgHandler()->iBarPortraitBitmapMask, invertMask);
            }
        else
            {
            // Toolbar is in landscape mode
            gc.BitBltMasked(Rect().iTl, iAppUi->SvgHandler()->iBarLandscapeBitmap, 
                    toolbarRect, iAppUi->SvgHandler()->iBarLandscapeBitmapMask, invertMask);
            }
        }
    else
        {
        // When terminal is in landscape mode
        if (Size().iHeight > Size().iWidth)
            {
            // Toolbar is in portrait mode
            gc.BitBltMasked(Rect().iTl, iAppUi->SvgHandler()->iCBAPortraitFatBitmap, 
                    toolbarRect, iAppUi->SvgHandler()->iCBAPortraitFatBitmapMask, invertMask);
            }
        else
            {
            // Toolbar is in landscape mode
            gc.BitBltMasked(Rect().iTl, iAppUi->SvgHandler()->iBarWideLandscapeBitmap, 
                    toolbarRect, iAppUi->SvgHandler()->iBarWideLandscapeBitmapMask, invertMask);
            }
        }
    

    // All buttons---------------------------------------------------
    TRect buttonRect = TRect(TPoint(0,0),TSize(KToolbarItemWidth,KToolbarItemHeight));

    // New ----------------------------------------------------------
    if (iTool == ENew)
        {
        gc.BitBltMasked(iNewPictureItem.iTl, iAppUi->SvgHandler()->iNew2ButtonBitmap, 
                buttonRect, iAppUi->SvgHandler()->iNew2ButtonBitmapMask, invertMask);
        }
    else
        {
        gc.BitBltMasked(iNewPictureItem.iTl, iAppUi->SvgHandler()->iNewButtonBitmap, 
                buttonRect, iAppUi->SvgHandler()->iNewButtonBitmapMask, invertMask);
        }
    // Pencil -------------------------------------------------------
    if (iTool == EPencil)
        {
        gc.BitBltMasked(iPencilItem.iTl, iAppUi->SvgHandler()->iPen2ButtonBitmap, 
                buttonRect, iAppUi->SvgHandler()->iPen2ButtonBitmapMask, invertMask);
        }
    else
        {
        gc.BitBltMasked(iPencilItem.iTl, iAppUi->SvgHandler()->iPenButtonBitmap, 
                buttonRect, iAppUi->SvgHandler()->iPenButtonBitmapMask, invertMask);
        }

    // Line -----------------------------------------------------------
    if (iTool == ELine)
        {
        gc.BitBltMasked(iLineItem.iTl, iAppUi->SvgHandler()->iLine2ButtonBitmap, 
                buttonRect, iAppUi->SvgHandler()->iLine2ButtonBitmapMask, invertMask);
        }
    else
        {
        gc.BitBltMasked(iLineItem.iTl, iAppUi->SvgHandler()->iLineButtonBitmap, 
                buttonRect, iAppUi->SvgHandler()->iLineButtonBitmapMask, invertMask);
        }

    // Circle -----------------------------------------------------------
    if (iTool == ECircle)
        {
        gc.BitBltMasked(iCircleItem.iTl, iAppUi->SvgHandler()->iCircle2ButtonBitmap, 
                buttonRect, iAppUi->SvgHandler()->iCircle2ButtonBitmapMask, invertMask);
        }
    else
        {
        gc.BitBltMasked(iCircleItem.iTl, iAppUi->SvgHandler()->iCircleButtonBitmap, 
                buttonRect, iAppUi->SvgHandler()->iCircleButtonBitmapMask, invertMask);
        }

    // Rect -----------------------------------------------------------
    if (iTool == ERect)
        {
        gc.BitBltMasked(iRectangleItem.iTl, iAppUi->SvgHandler()->iRect2ButtonBitmap, 
                buttonRect, iAppUi->SvgHandler()->iRect2ButtonBitmapMask, invertMask);
        }
    else
        {
        gc.BitBltMasked(iRectangleItem.iTl, iAppUi->SvgHandler()->iRectButtonBitmap, 
                buttonRect, iAppUi->SvgHandler()->iRectButtonBitmapMask, invertMask);
        }

    // Color -----------------------------------------------------------
    if (iTool == EColor)
        {
        gc.BitBltMasked(iColorItem.iTl, iAppUi->SvgHandler()->iColor2ButtonBitmap, 
                buttonRect, iAppUi->SvgHandler()->iColor2ButtonBitmapMask, invertMask);
        }
    else
        {
        gc.BitBltMasked(iColorItem.iTl, iAppUi->SvgHandler()->iColorButtonBitmap, 
                buttonRect, iAppUi->SvgHandler()->iColorButtonBitmapMask, invertMask);
        }
    // Draw selected color rect
    TRect colorRect = iColorItem;
    colorRect.Shrink(13,13);
    colorRect.Move(6,-6);
    gc.SetBrushColor(iParent->DrawingEngine()->Color());
    gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
    gc.DrawRect(colorRect);
    gc.SetBrushStyle(CGraphicsContext::ENullBrush);

    
    // Size -----------------------------------------------------------
    if (iPenButtonIsUp)
        {
        // Button is UP
        if (iPenButtonIndex == 1)
            {
            gc.BitBltMasked(iSizeItem.iTl, iAppUi->SvgHandler()->iSize1ButtonBitmap, 
                    buttonRect, iAppUi->SvgHandler()->iSize1ButtonBitmapMask, invertMask);
            }
        else if (iPenButtonIndex == 2)
            {
            gc.BitBltMasked(iSizeItem.iTl, iAppUi->SvgHandler()->iSize2ButtonBitmap, 
                    buttonRect, iAppUi->SvgHandler()->iSize2ButtonBitmapMask, invertMask);
            }
        else if (iPenButtonIndex == 3)
            {
            gc.BitBltMasked(iSizeItem.iTl, iAppUi->SvgHandler()->iSize3ButtonBitmap, 
                    buttonRect, iAppUi->SvgHandler()->iSize3ButtonBitmapMask, invertMask);
            }
        }
    else
        {
        // Button is DOWN
        if (iPenButtonIndex == 1)
            {
            gc.BitBltMasked(iSizeItem.iTl, iAppUi->SvgHandler()->iSize1DownButtonBitmap, 
                    buttonRect, iAppUi->SvgHandler()->iSize1DownButtonBitmapMask, invertMask);
            }
        else if (iPenButtonIndex == 2)
            {
            gc.BitBltMasked(iSizeItem.iTl, iAppUi->SvgHandler()->iSize2DownButtonBitmap, 
                    buttonRect, iAppUi->SvgHandler()->iSize2DownButtonBitmapMask, invertMask);
            }
        else if (iPenButtonIndex == 3)
            {
            gc.BitBltMasked(iSizeItem.iTl, iAppUi->SvgHandler()->iSize3DownButtonBitmap, 
                    buttonRect, iAppUi->SvgHandler()->iSize3DownButtonBitmapMask, invertMask);
            }

        // Draw selected color in down button
        colorRect = iSizeItem;
        if (iPenButtonIndex == 1)
            {
            colorRect.Shrink(8,21);
            colorRect.Move(1,-15);
            }
        else if (iPenButtonIndex == 2)
            {
            colorRect.Shrink(8,18);
            colorRect.Move(1,-5);
            }
        else if (iPenButtonIndex == 3)
            {
            colorRect.Shrink(8,16);
            colorRect.Move(1,10);
            }
        gc.SetBrushStyle(CGraphicsContext::ESolidBrush);
        gc.DrawRect(colorRect);
        gc.SetBrushStyle(CGraphicsContext::ENullBrush);
        }
    
    
    // Undo -----------------------------------------------------------
    if (iTool == EUndo)
        {
        gc.BitBltMasked(iUndoItem.iTl, iAppUi->SvgHandler()->iUndo2ButtonBitmap, 
                buttonRect, iAppUi->SvgHandler()->iUndo2ButtonBitmapMask, invertMask);
        }
    else
        {
        gc.BitBltMasked(iUndoItem.iTl, iAppUi->SvgHandler()->iUndoButtonBitmap, 
                buttonRect, iAppUi->SvgHandler()->iUndoButtonBitmapMask, invertMask);
        }

    }

void CToolbarControl::SizeChanged()
    {
    // Calculate new item positions
    TRect item = Rect();
    
    // When terminal is in portrait mode
    if (iAppUi->CbaPosition() == AknLayoutUtils::EAknCbaLocationBottom)
        {
        if (Size().iHeight > Size().iWidth)
            {
            // Toolbar is in portrait mode
            item.SetWidth(KToolbarItemWidth);
            item.SetHeight(KToolbarItemHeight);

            // Move to toolbar center
            item.Move((KPortraitToolbarWidth-KToolbarItemWidth)/2,0);
            
            item.Move(0,KToolbarItemGap);
            iNewPictureItem = item;
            
            item.Move(0,KToolbarItemHeight+KToolbarItemGap);
            iPencilItem = item;
            
            item.Move(0,KToolbarItemHeight+KToolbarItemGap);
            iLineItem = item;

            item.Move(0,KToolbarItemHeight+KToolbarItemGap);
            iCircleItem = item;

            item.Move(0,KToolbarItemHeight+KToolbarItemGap);
            iRectangleItem = item;

            item.Move(0,KToolbarItemHeight+KToolbarItemGap);
            iColorItem = item;

            item.Move(0,KToolbarItemHeight+KToolbarItemGap);
            iSizeItem = item;

            item.Move(0,KToolbarItemHeight+KToolbarItemGap);
            iUndoItem = item;
            }
        else
            {
            // Toolbar is in landscape mode
            item.SetWidth(KToolbarItemWidth);
            item.SetHeight(KToolbarItemHeight);

            // Move to toolbar center
            item.Move(0,(KLandscapeToolbarHeight-KToolbarItemHeight)/2);
            
            item.Move(KToolbarItemGap,0);
            iNewPictureItem = item;

            item.Move(KToolbarItemWidth+KToolbarItemGap,0);
            iPencilItem = item;

            item.Move(KToolbarItemWidth+KToolbarItemGap,0);
            iLineItem = item;

            item.Move(KToolbarItemWidth+KToolbarItemGap,0);
            iCircleItem = item;

            // New item line ------
            item = Rect();
            item.SetWidth(KToolbarItemWidth);
            item.SetHeight(KToolbarItemHeight);
            item.Move(0,KToolbarItemHeight+KToolbarItemGap +((KPortraitToolbarWidth - KToolbarItemWidth)/2));

            item.Move(KToolbarItemGap,0);
            iRectangleItem = item;
            
            item.Move(KToolbarItemWidth+KToolbarItemGap,0);
            iColorItem = item;

            item.Move(KToolbarItemWidth+KToolbarItemGap,0);
            iSizeItem = item;

            item.Move(KToolbarItemWidth+KToolbarItemGap,0);
            iUndoItem = item;
            }
        }
    else
        {
        // When terminal is in landscape mode
        if (Size().iHeight > Size().iWidth)
            {
            // Toolbar is in portrait mode
            item.SetWidth(KToolbarItemWidth);
            item.SetHeight(KToolbarItemHeight);

            // Move to toolbar center
            item.Move((KPortraitToolbarWidth2Rows-KToolbarItemWidth-KToolbarItemWidth)/2,0);
            
            item.Move(0,KToolbarItemGap);
            iNewPictureItem = item;
            
            item.Move(0,KToolbarItemWidth+KToolbarItemGap);
            iPencilItem = item;

            item.Move(0,KToolbarItemWidth+KToolbarItemGap);
            iLineItem = item;

            item.Move(0,KToolbarItemWidth+KToolbarItemGap);
            iCircleItem = item;

            // New item line ------
            item = Rect();
            item.SetWidth(KToolbarItemWidth);
            item.SetHeight(KToolbarItemHeight);
            item.Move(KToolbarItemWidth+KToolbarItemGap +((KPortraitToolbarWidth2Rows-KToolbarItemWidth-KToolbarItemWidth)/2),0);
            
            item.Move(0,KToolbarItemGap);
            iRectangleItem = item;
            
            item.Move(0,KToolbarItemWidth+KToolbarItemGap);
            iColorItem = item;

            item.Move(0,KToolbarItemWidth+KToolbarItemGap);
            iSizeItem = item;

            item.Move(0,KToolbarItemWidth+KToolbarItemGap);
            iUndoItem = item;
            }
        else
            {
            // Toolbar is in landscape mode
            item.SetWidth(KToolbarItemWidth);
            item.SetHeight(KToolbarItemHeight);

            // Move to toolbar center
            item.Move(0,(KLandscapeToolbarHeight-KToolbarItemHeight)/2);
            
            item.Move(KToolbarItemGap,0);
            iNewPictureItem = item;

            item.Move(KToolbarItemWidth+KToolbarItemGap,0);
            iPencilItem = item;

            item.Move(KToolbarItemWidth+KToolbarItemGap,0);
            iLineItem = item;

            item.Move(KToolbarItemWidth+KToolbarItemGap,0);
            iCircleItem = item;

            item.Move(KToolbarItemWidth+KToolbarItemGap,0);
            iRectangleItem = item;
            
            item.Move(KToolbarItemWidth+KToolbarItemGap,0);
            iColorItem = item;

            item.Move(KToolbarItemWidth+KToolbarItemGap,0);
            iSizeItem = item;

            item.Move(KToolbarItemWidth+KToolbarItemGap,0);
            iUndoItem = item;
            }
        }
    }


void CToolbarControl::Move(const TPoint& aPoint)
    {
    // User is moving/dragging this toolbar
    TPoint offset;
    offset.iX =  aPoint.iX - iMovePoint.iX;
    offset.iY = aPoint.iY - iMovePoint.iY;
    
    TRect rect = Rect();
    rect.Move(offset);
    
    if (rect.iTl.iX >= KToolbarGap && rect.iTl.iY >= KToolbarGap &&
        rect.iBr.iY <= iParent->Size().iHeight - KToolbarGap && rect.iBr.iX <= iParent->Size().iWidth - KToolbarGap)
        {
        // Move
        SetRect(rect);
        }
    else
        {
        if (rect.iTl.iY < KToolbarGap)
            {
            SetTop();
            }
        else if (rect.iBr.iY > iParent->Size().iHeight - KToolbarGap)
            {
            SetBottom();
            }
        else if (rect.iBr.iX > iParent->Size().iWidth - KToolbarGap)
            {
            SetRight();
            }
        else if (rect.iTl.iX < KToolbarGap)
            {
            SetLeft();
            }
        }
    }

void CToolbarControl::SetTop()
    {
    // Set toolbar location in screen
    TRect rect;
    if (iAppUi->CbaPosition() != AknLayoutUtils::EAknCbaLocationBottom)
        {
        // Set toolbar to TOP in terminal landscape mode
        rect = TRect(TPoint((iParent->Size().iWidth - KToolbarItemWidth*KToolbarItemCount)/2,KToolbarGap),
                TSize(KToolbarItemWidth*KToolbarItemCount + KToolbarItemGap*(KToolbarItemCount+1),KLandscapeToolbarHeight));

        }
    else
        {
        // Set toolbar to TOP in terminal portrait mode
        TInt count = KToolbarItemCount/2;
        rect = TRect(TPoint((iParent->Size().iWidth - KToolbarItemWidth*count)/2,KToolbarGap),
                TSize(KToolbarItemWidth*count + KToolbarItemGap*(count),KLandscapeToolbarHeight2Rows));
        }
    
    SetRect(rect);
    iTouchFeedBack->InstantFeedback(ETouchFeedbackBasic);
    }

void CToolbarControl::SetBottom()
    {
    // Set toolbar location in screen
    TRect rect;
    if (iAppUi->CbaPosition() != AknLayoutUtils::EAknCbaLocationBottom)
        {
        // Set toolbar to BOTTOM in terminal landscape mode
        rect = TRect(TPoint((iParent->Size().iWidth - KToolbarItemWidth*KToolbarItemCount)/2,iParent->Size().iHeight - KToolbarGap - KLandscapeToolbarHeight),
                TSize(KToolbarItemWidth*KToolbarItemCount + KToolbarItemGap*(KToolbarItemCount+1),KLandscapeToolbarHeight));

        SetRect(rect);
        iTouchFeedBack->InstantFeedback(ETouchFeedbackBasic);
        }
    else
        {
        // Terminal is in portrait mode
        SetTop();
        }
    }

void CToolbarControl::SetRight()
    {
    // Set toolbar location in screen
    if (iAppUi->CbaPosition() != AknLayoutUtils::EAknCbaLocationBottom)
        {
        // Terminal is in landscape mode
        SetTop();
        }
    else
        {
        // Set toolbar to RIGHT in terminal portrait mode
        TRect rect = TRect(TPoint(iParent->Size().iWidth - KPortraitToolbarWidth - KToolbarGap, (iParent->Size().iHeight - KToolbarItemWidth*KToolbarItemCount)/2),
                TSize(KPortraitToolbarWidth,KToolbarItemHeight*KToolbarItemCount + KToolbarItemGap*(KToolbarItemCount+1)));
        SetRect(rect);
        iTouchFeedBack->InstantFeedback(ETouchFeedbackBasic);
        }
    }

void CToolbarControl::SetLeft()
    {
    // Set toolbar location in screen
    TRect rect;
    if (iAppUi->CbaPosition() != AknLayoutUtils::EAknCbaLocationBottom)
        {
        // Set toolbar to LEFT in terminal landscape mode
        TInt count = KToolbarItemCount/2;
        rect = TRect(TPoint(KToolbarGap,(iParent->Size().iHeight - KToolbarItemWidth*count)/2),
                TSize(KPortraitToolbarWidth2Rows,KToolbarItemHeight*count + KToolbarItemGap*(count)));
        }
    else
        {
        // Set toolbar to LEFT in terminal portrait mode
        rect = TRect(TPoint(KToolbarGap,(iParent->Size().iHeight - KToolbarItemWidth*KToolbarItemCount)/2),
                TSize(KPortraitToolbarWidth,KToolbarItemHeight*KToolbarItemCount + KToolbarItemGap*(KToolbarItemCount+1)));
        }

    SetRect(rect);
    iTouchFeedBack->InstantFeedback(ETouchFeedbackBasic);
    }

TBool CToolbarControl::IsDragDone(const TPointerEvent& aPointerEvent)
    {
    // We accept dragging when user has done at least KDraggingAccepted pixel drag
    TBool ret = EFalse;
    
    TInt x =  Abs(aPointerEvent.iPosition.iX - iDragStartPoint.iX);
    TInt y = Abs(aPointerEvent.iPosition.iY - iDragStartPoint.iY);

    if (x > KDraggingAccepted || y > KDraggingAccepted)
        {
        ret  = ETrue;
        }
    
    return ret;
    }

void CToolbarControl::HandlePointerEventL(const TPointerEvent& aPointerEvent)
    {
    if (aPointerEvent.iType == TPointerEvent::EButton1Down)
        {
        iDrag = EFalse;
        iMovePoint = aPointerEvent.iPosition;
        iDragStartPoint = aPointerEvent.iPosition;
        }
    else if (aPointerEvent.iType == TPointerEvent::EDrag)
        {
        // Do we accept dragging?
        if (!iDrag && IsDragDone(aPointerEvent))
            {
            iDrag = ETrue;
            }

        // Dragging accepted in IsDragDone()
        if (iDrag)
            {
            TRect acceptedArea = Rect();
            // Grow a little area of where dragging is accepted
            acceptedArea.Grow(KDraggingAcceptGrow,KDraggingAcceptGrow);
            if (acceptedArea.Contains(aPointerEvent.iPosition))
                {
                Move(aPointerEvent.iPosition);
                iMovePoint = aPointerEvent.iPosition;
                }
            }
        }
    else if (aPointerEvent.iType == TPointerEvent::EButton1Up)
        {
        if (!iDrag)
            {
            // Handle tool selection
            if (iNewPictureItem.Contains(aPointerEvent.iPosition))
                {
                iTouchFeedBack->InstantFeedback(ETouchFeedbackSensitive);
                iParent->NewPictureL();
                }
            else if (iPencilItem.Contains(aPointerEvent.iPosition))
                {
                iTouchFeedBack->InstantFeedback(ETouchFeedbackSensitive);
                iTool = EPencil;
                }
            else if (iLineItem.Contains(aPointerEvent.iPosition))
                {
                iTouchFeedBack->InstantFeedback(ETouchFeedbackSensitive);
                iTool = ELine;
                }
            else if (iCircleItem.Contains(aPointerEvent.iPosition))
                {
                iTouchFeedBack->InstantFeedback(ETouchFeedbackSensitive);
                iTool = ECircle;
                }
            else if (iRectangleItem.Contains(aPointerEvent.iPosition))
                {
                iTouchFeedBack->InstantFeedback(ETouchFeedbackSensitive);
                iTool = ERect;
                }
            else if (iColorItem.Contains(aPointerEvent.iPosition))
                {
                iTouchFeedBack->InstantFeedback(ETouchFeedbackSensitive);
                SelectColourL();
                }
            else if (iSizeItem.Contains(aPointerEvent.iPosition))
                {
                iTouchFeedBack->InstantFeedback(ETouchFeedbackSensitive);
                SelectSizeL();
                }
            else if (iUndoItem.Contains(aPointerEvent.iPosition))
                {
                iTouchFeedBack->InstantFeedback(ETouchFeedbackSensitive);
                iParent->RestoreBackBufferL();
                }
            }
        iDrag = EFalse;
        }
    }

void CToolbarControl::DrawSizeButton() const
    {
    DrawNow(iSizeItem);
    }

void CToolbarControl::ShowUnpressedSizeButtonL()
    {
    // This shows unpressed size-button after some period (=KSizebuttonDownTime)
    if (iPeriodic)
        {
        iPeriodic->Cancel();
        }
    else
        {
        iPeriodic = CPeriodic::NewL(CActive::EPriorityIdle);
        }
    iPeriodic->Start(KSizebuttonDownTime, KSizebuttonDownTime, TCallBack(SizeButtonUp, this));
    }

TInt CToolbarControl::SizeButtonUp(TAny* aAny)
    {
    // Setting size button back to up state
    
    CToolbarControl* self = static_cast<CToolbarControl*>( aAny );

    // Cancel timer
    self->iPeriodic->Cancel();

    // Set button back to UP
    self->iPenButtonIsUp = ETrue;
    
    // Draw button again
    self->DrawSizeButton();
    
    // Give sensitive vibra feedback when button back to up
    self->iTouchFeedBack->InstantFeedback(ETouchFeedbackSensitive);
    
    return KErrNone;
    }

void CToolbarControl::SelectSizeL()
    {
    // Set pen size
    if (iPenSize < KMAXPenSize)
        {
        iPenSize = iPenSize+2;
        }
    else
        {
        iPenSize = KMINPenSize;
        }
    iParent->DrawingEngine()->SetPenSize(TSize(iPenSize,iPenSize));
    
    // Set pen button
    switch (iPenSize)
        {
        case 2:
            {
            iPenButtonIndex = 1;
            break;
            }
        case 4:
            {
            iPenButtonIndex = 2;
            break;
            }
        case 6:
            {
            iPenButtonIndex = 3;
            break;
            }
        default:
            {
            iPenButtonIndex = 1;
            break;
            }
        };
    
    // Show selected size button
    iPenButtonIsUp = EFalse; // Button is DOWN now
    DrawSizeButton();
    
    // Draw unpressed state again after 50ms 
    ShowUnpressedSizeButtonL();
    }

void CToolbarControl::SelectColourL()
    {
    TBool noneExist = ETrue; 
    TBool noneChosen = ETrue; 
    
    // Set current color
    TRgb colour = iParent->DrawingEngine()->Color(); 

    // Build color array for dialog
    CArrayFixFlat<TRgb>* colours = new(ELeave) CArrayFixFlat<TRgb>(6);
    colours->AppendL(KRgbBlack); 
    colours->AppendL(KRgbDarkGray); 
    colours->AppendL(KRgbDarkRed); 
    colours->AppendL(KRgbDarkGreen); 
    colours->AppendL(KRgbDarkYellow); 
    colours->AppendL(KRgbDarkBlue);
    colours->AppendL(KRgbDarkMagenta);
    colours->AppendL(KRgbDarkCyan);
    colours->AppendL(KRgbRed);
    colours->AppendL(KRgbGreen);
    colours->AppendL(KRgbYellow);
    colours->AppendL(KRgbBlue);
    colours->AppendL(KRgbMagenta);
    colours->AppendL(KRgbCyan);
    colours->AppendL(KRgbGray);
    colours->AppendL(KRgbWhite);

    // Show color selection dialog
    CAknColourSelectionGrid *d = CAknColourSelectionGrid::NewL( colours, noneExist, noneChosen, colour);
    d->ExecuteLD();
    delete colours;

    // Activate color
    iParent->DrawingEngine()->SetColor(colour);
    }

